using System;
using System.IO;

namespace AVIInfo
{
	/// <summary>
	/// Summary description for Options.
	/// </summary>
	public class Options
	{
        public bool bKeyFrames = false;
        public string sFileName = "";
        public bool bValid = false;
        public bool bShowHelp = false;
        public bool bBasicInfo = false;
        public bool bAdvancedInfo = false;
        public bool bDiagnostics = false;
        private bool bNextIsBSIFile = false;
        public bool bUseBSIFile = false;
        public bool bWarnEndKeyframe = false;
        public bool bVersion = false;

        public string sBSIFile;
        public string sErrors;
        public string sWarnings;

        public Options(string[] args)
        {
            SetOptions(args);
        }

        private void SetDefaults()
        {
            sErrors = "";
            sWarnings = "";
            bKeyFrames = false;
            sFileName = "";
            bValid = false;
            bShowHelp = false;
            bBasicInfo = false;
            bAdvancedInfo = false;
            bDiagnostics = false;
            bNextIsBSIFile = false;
            bUseBSIFile = false;
            sBSIFile = "";
            bWarnEndKeyframe = false;
            bVersion = false;
        }

        public void SetOptions(string[] args)
        {
            SetDefaults();
            int iArg = 0;

            string sArg;

            for (int i = 0; i < args.Length; i++)
            {
                sArg = args[i];

                if ((sArg[0] == '-') || (sArg[0] == '/'))
                {
                    ProcessCharArgs(sArg.Substring(1));
                }
                else
                {
                    if (bNextIsBSIFile)
                    {
                        sBSIFile = sArg;
                        bUseBSIFile = true;
                        bNextIsBSIFile = false;
                    }
                    else
                    {
                        switch(iArg)
                        {
                            case 0:
                                sFileName = sArg;
                                break;
                            default:
                                AddArgWarning(sArg);
                                break;
                        }

                        iArg++;
                    }
                }
            }

            bValid = true;

            if ((sFileName == "") && (!bUseBSIFile))
            {
                AddError("No filename specified");
            }

            if (bNextIsBSIFile)
            {
                AddError("The -b option was specified, but was not followed by a filename.");
            }

            if (Errors)
                bValid = false;
        }

        private string NextArg(string[] args, ref int iIndex)
        {
            if (iIndex + 1 >= args.Length)
                return "";

            iIndex++;
            return args[iIndex];
        }

        private void ProcessCharArgs(string sArg)
        {
            foreach(char c in sArg)
            {
                switch(c)
                {
                    case 'b':
                    case 'B':
                        bNextIsBSIFile = true;
                        break;
                    case 'k':
                    case 'K':
                        bKeyFrames = true;
                        break;
                    case 'd':
                    case 'D':
                        bDiagnostics = true;
                        break;
                    case 'i':
                    case 'I':
                        bBasicInfo = true;
                        break;
                    case 'o':
                    case 'O':
                        bAdvancedInfo = true;
                        break;
                    case 'v':
                    case 'V':
                        bVersion = true;
                        break;
                    case 'w':
                    case 'W':
                        bWarnEndKeyframe = true;
                        break;
                    case '?':
                    case 'h':
                    case 'H':
                        bShowHelp = true;
                        break;
                    default:
                        AddOptionWarning(c);
                        break;
                }
            }
        }

        private void AddArgWarning(string sArg)
        {
            AddWarning(string.Format("Ignoring unknown argument \"{0}\"", sArg));
        }

        private void AddOptionWarning(char c)
        {
            AddWarning(string.Format("Ignoring unknown option '{0}'", c));
        }

        private void AddWarning(string s)
        {
            sWarnings += "Warning: " + s + "\n";
        }

        private void AddError(string s)
        {
            sErrors += "Error: " + s + "\n";
        }

        public bool Warnings
        {
            get
            {
                return (sWarnings != "");
            }
        }

        public bool Errors
        {
            get
            {
                return (sErrors != "");
            }
        }

        public static string Usage
        {
            get
            {
                return string.Format(
                    "Usage:    {0} [-dikovw] [-b BSIFile] AVIFile\n" +
                    "\n" +
                    "Options:  -b  Adjust a BSI file to match the AVI keyframes\n" +
                    "          -d  Print diagnostic/debug information\n" +
                    "          -i  Print AVI basic information\n" +
                    "          -k  Print a list of keyframes in AVIFile\n" + 
                    "          -o  Print AVI offset information\n" +
                    "          -v  Print the version number\n" +
                    "          -w  Warn if the last frame is a keyframe\n" +

                    "",
                    Path.GetFileNameWithoutExtension(System.Diagnostics.Process.GetCurrentProcess().MainModule.FileName)
                    );
            }
        }

    }
}
